﻿define([
    // Dojo
    "dojo",
    "dojo/topic",
    "dojo/_base/declare",
    "dijit/registry",
    // Dijit
    "dijit/_TemplatedMixin",
    "dijit/_Container",
    "dijit/layout/_LayoutWidget",
    //"dijit/layout/_ContentPane",
    "dijit/_WidgetsInTemplateMixin",
    "epi-cms/_ContentContextMixin",
    "epi/dependency",
    "epi/routes",
    //Custom
    "workspace/components/WorkspaceGrid",
    "workspace/commands/AddCurrentItem",
    "workspace/commands/ClearItems",
    "workspace/commands/Settings",
    "workspace/commands/DashboardCommand",
    "workspace/commands/AssetErrorLogsCommand",
    "epi/shell/command/_WidgetCommandProviderMixin",
    "dojo/text!./Workspaces.html",
    "dojo/text!./lion-modal.html"
], function (
    // Dojo
    dojo,
    topic,
    declare,
    registry,
    // Dijit
    _TemplatedMixin,
    _Container,
    _LayoutWidget,
    _WidgetsInTemplateMixin,
    _ContentContextMixin,
    dependency,
    routes,

    WorkspaceGrid,
    AddCurrentItem,
    ClearItems,
    Settings,
    DashboardCommand,
    AssetErrorLogsCommand,
    _WidgetCommandProviderMixin,
    template,
    templateTest
) {

        return declare(
            [_Container, _LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ContentContextMixin, _WidgetCommandProviderMixin], {
                templateString: template,
                widgetsInTemplate: true,
                store: null,
                context: null,
                authenticationstore: null,

                contextChanged: function (context, callerData) {
                    this.context = context;
                    this.inherited(arguments);
                },

                _getRestPath: function (name) {
                    return routes.getRestPath({ moduleArea: "lionbridgeconnector", storeName: name }); //moduleArea: 'app' for internal build | lionbridgeconnector for external build
                },
                //_publishNow: function() {
                //    this.store.put('-2');
                //    this._reloadGrid();
                //},
                _store: function () {
                    var registry = dependency.resolve("epi.storeregistry");
                    if (registry._stores["workspace.workspacestore"] === undefined) {
                        registry.create("workspace.workspacestore", this._getRestPath("workspacestore"));
                        registry.create("workspace.translationprojectstore", this._getRestPath("translationprojectstore"));
                        registry.create("workspace.translationprojecteditorstore", this._getRestPath("translationprojecteditorstore"));
                        registry.create("workspace.translationprojecttreestore", this._getRestPath("translationprojecttreestore"));
                        registry.create("workspace.translationprojectblocktreestore", this._getRestPath("translationprojectblocktreestore"));
                        registry.create("workspace.translationsearchitemsstore", this._getRestPath("translationsearchitemsstore"));
                        registry.create("workspace.publishprojectstore", this._getRestPath("publishprojectstore"));
                        registry.create("workspace.importprojectstore", this._getRestPath("importprojectstore"));
                        registry.create("workspace.settingstore", this._getRestPath("settingstore"));
                        registry.create("workspace.archiveprojectstore", this._getRestPath("archiveprojectstore"));
                        registry.create("workspace.dashboardstore", this._getRestPath("dashboardstore"));
                        registry.create("workspace.publishprojectlanguagestore", this._getRestPath("publishprojectlanguagestore"));
                        registry.create("workspace.importprojectlanguagestore", this._getRestPath("importprojectlanguagestore"));
                        registry.create("workspace.authenticationstore", this._getRestPath("authenticationstore"));
                        registry.create("workspace.asseterrorlogsstore", this._getRestPath("asseterrorlogsstore"));
                        registry.create("workspace.translationprojectasseterrorlogsliststore", this._getRestPath("translationprojectasseterrorlogsliststore"));
                    }

                    this.store = this.store || registry.get("workspace.workspacestore");
                    this.authenticationstore = this.authenticationstore || registry.get("workspace.authenticationstore");

                    LionBridgeConnectorGadget.dependency = dependency;
                    LionBridgeConnectorGadget.dashboardstore = registry.get("workspace.dashboardstore")

                    return this.store;
                },

                _reloadGrid: function () {
                    this.contentQuery.set("queryParameters", { workspace: 1, page: 1 });
                    this.contentQuery.set("queryName", "WorkspaceQuery");
                },

                startup: function () {
                    var self = this;

                    self.inherited(arguments);
                    this._store();

                    this.contentQuery.store = this.store;

                    LionBridgeConnectorGadget.dojo = dojo;
                    LionBridgeConnectorGadget.topic = topic;

                    if (typeof LionBridgeConnectorGadget !== "undefined") {
                        LionBridgeConnectorGadget.contentQuery = this.contentQuery;
                    }

                    this._reloadGrid();

                    dojo.when(self.authenticationstore.get("1"),
                        function (item) {

                            dojo.body().insertAdjacentHTML("beforeEnd", templateTest);

                            if (item.result) {
                                self.set("commands",
                                    [
                                        new AddCurrentItem({ store: self.store, contentQuery: self.contentQuery }),
                                        new DashboardCommand({ typeIdentifiers: self.typeIdentifiers }),
                                        new AssetErrorLogsCommand({ typeIdentifiers: self.typeIdentifiers }),
                                        new Settings({ typeIdentifiers: self.typeIdentifiers })
                                    ]);
                            } else {
                                self.set("commands",
                                    [
                                        new AddCurrentItem({ store: self.store, contentQuery: self.contentQuery }),
                                        new DashboardCommand({ typeIdentifiers: self.typeIdentifiers }),
                                    ]);
                            }
                        });
                },
                postCreate: function () {
                    this._store();
                },
            });
    });

var LionBridgeConnectorGadget = LionBridgeConnectorGadget || (function () {

    var contentQuery;
    var dependency;
    var dashboardstore;
    var dojo;
    var topic;
    function reloadGadget() {
        this.contentQuery.set("queryParameters", { workspace: 1, page: 1 });
        this.contentQuery.set("queryName", "WorkspaceQuery");
    }

    function openDashboard() {
        var registry = LionBridgeConnectorGadget.dependency.resolve("epi.storeregistry");
        var store = registry.get("workspace.dashboardstore");

        LionBridgeConnectorGadget.dojo.when(store.get(),
            function (item) {
                LionBridgeConnectorGadget.topic.publish("/epi/shell/context/request",
                    { uri: "epi.cms.contentdata:///" + item.pageRef },
                    {
                        sender: this,
                        viewName: this.view,
                        typeIdentifiers: this.typeIdentifiers,
                        forceReload: true,
                        forceContextChange: true
                    });

                window.location.href = "#context=epi.cms.contentdata:///" + item.pageRef;
                window.location.reload(true);//.reload();
            });
    }

    return {
        contentQuery: contentQuery,
        dependency: dependency,
        reloadGadget: reloadGadget,
        openDashboard: openDashboard,
        dashboardstore: dashboardstore,
        dojo: dojo,
        topic: topic,
    }

})();
