﻿define([
    "dojo/_base/declare",
        "epi/dependency",
            "epi/routes",
    "epi/shell/widget/dialog/Alert",
    "epi/shell/widget/dialog/Confirmation",
    "epi/shell/command/_Command"
], function (declare, dependency, routes, Alert, Confirmation, _Command) {

    return declare([_Command], {
        store: null,
        contentQuery: null,

        label: "Remove Project",

        category: "context",
        iconClass: "epi-iconClose",
        canExecute: true,

        _execute: function () {
            var contentLink = this.model.contentLink;
            var x = this.contentQuery;
            var confirmation = new Confirmation({
                title: "Remove project",
                description: "Are you sure you want to remove the project?",
                onAction: function (confirmed) {
                    if (confirmed) {
                        $('.loadingDiv').fadeIn(500);
                        var registry = dependency.resolve("epi.storeregistry");
                        this.store = registry.get("workspace.workspacestore");
                        this.store.remove(contentLink).then(function (item) {
                            if (item != undefined && item.error) {
                                var errorDialog = new Alert({
                                    heading: "Remove Project",
                                    description: "Something went wrong when removing the project due to: " + item.errorMessage
                                });
                                errorDialog.show();
                            } else {
                                x.set("queryParameters", { workspace: 1, page: 1 });
                                x.set("queryName", "WorkspaceQuery");

                                if (typeof LionBridgeConnectorGrid !== "undefined") {
                                    LionBridgeConnectorGrid.onReloadGrid();
                                }
                            }

                            $('.loadingDiv').fadeOut(500);
                        });
                    }
                }
            });
            confirmation.show();
        },

        _onModelChange: function () {
            // summary:
            //      Updates canExecute after the model has been updated. Subclasses should override this method.
            // tags:
            //      protected
            var sendLink = this;
            var registry = dependency.resolve("epi.storeregistry");
            var store = registry.get("workspace.translationprojectstore");
            dojo.when(store.get(this.model.contentLink + "?" + Math.random() * 9999999999), function (item) {
                
                if (item.status == 2) {
                    sendLink.set("canExecute", false);
                } else {
                    sendLink.set("canExecute", true);
                }
            });
        }
    });
})