﻿define("workspace/components/WorkspaceGrid", [
    "dojo",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "epi",
    "epi/dependency",
    "epi-cms/component/ContentQueryGrid",
    "epi-cms/dgrid/formatters",
    "workspace/commands/RemoveItem",
    "workspace/commands/SendProject",
    "workspace/commands/PublishItem",
    "workspace/commands/CancelProject",
    "workspace/commands/ReadClayTablet",
    "workspace/commands/CloneJob",
], function (dojo, declare, lang, epi, dependency, ContentQueryGrid, formatters, RemoveItem, SendProject, PublishItem, CancelProject, ReadClayTablet, CloneJob) {

    return declare([ContentQueryGrid], {
        constructor: function () {

            var self = this;
            
        },
        sortKey: null,
        descending: false,
        postMixInProperties: function () {
            this.inherited(arguments);

            //encapsulate the epi-server menu formatter to set the tooltip on the context menu
            var customFormatter = function (data, object) {
                
                var menu = formatters.menu({ title: "Options"});
                return menu;
            };

            lang.mixin(this.defaultGridMixin, {
                columns: {
                    name: {
                        renderCell: function (object, value, node) {
                            var nameValue = value;
                            var valueSplit = value.split("|___|");
                            if (valueSplit.length > 0 && valueSplit.length === 5) {
                                nameValue = valueSplit[0];
                            }
                            node.innerHTML = formatters.contentItem(object.typeIdentifier, object.missingLanguageBranch, nameValue);
                        }
                    },
                    status: {
                        className: "epi-width25",
                        renderCell: function (object, value, node) {
                            
                            var valueSplit = object.name.split("|___|");
                            if (valueSplit.length > 0 && valueSplit.length === 5) {
                                var item = {
                                    percentageCompleted: valueSplit[1],
                                    status: valueSplit[2],
                                    error: valueSplit[3],
                                    errorMessage: valueSplit[4],
                                }
                            //}
                            //var registry = dependency.resolve("epi.storeregistry");
                            //var store = registry.get("workspace.translationprojectstore");

                            //var query = store.get(object.contentLink + "?" + Math.random() * 9999999999);

                            //if (query.then) {
                            //    query.then(function (item) {

                                    var cssClass = item.status === "3" ? "epi-iconCheckmark" : "epi-iconClose";
                                    switch (item.status) {
                                        case "0":
                                            cssClass = "epi-iconShare";
                                            break;
                                        case "1":
                                        case "101":
                                        case "102":
                                            cssClass = "epi-iconClock";
                                            break;
                                        case "2":
                                            cssClass = "epi-iconDownload";
                                            break;
                                        case "3":
                                            cssClass = "epi-iconCheckmark epi-icon--success";
                                            break;
                                        default:
                                            break;
                                    }

                                    var title = "";

                                    if (item.error !== "False") {
                                        title = 'Something went wrong: ' + item.errorMessage;
                                        cssClass = "epi-iconError";
                                    }
                                    
                                    var percentage = item.percentageCompleted === 0 ? '' : item.percentageCompleted + '%';
                                    node.innerHTML = "<span id=\"exampleNode\" class=\"dijitInline dijitIcon " + cssClass + "\" title=\"" + title + "\"></span><span>" + percentage + " </span>";
                                //});
                            } else {
                                node.innerHTML = "nada";
                            }
                        }
                    },
                    menu: {
                        className: "epi-width10",
                        formatter: customFormatter
                    }
                }
            });

            this.commands = [
                new SendProject({ store: this.store, contentQuery: this }),
                new ReadClayTablet({ store: this.store, contentQuery: this }),
                new PublishItem({ store: this.store, contentQuery: this }),
                new CancelProject({ store: this.store, contentQuery: this }),
                new RemoveItem({ store: this.store, contentQuery: this }),
                //new CloneJob({ store: this.store, contentQuery: this })
            ];
        },

        fetchData: function () {
            var queryOptions = { ignore: ["query"] };
            if (this.sortKey) {
                queryOptions.sort = [{ attribute: this.sortKey, descending: this.descending }];
            }
            this.grid.set("queryOptions", queryOptions);

            var queryParameters = this.queryParameters || {};
            queryParameters.query = this.queryName;
            this.grid.set("query", queryParameters);

        },

        postCreate: function () {
            this.inherited(arguments);
            this.grid.contextMenu.addProvider(this);
        },

        _onSelect: function (e) {
            var item = e.rows[0].data;

            this.commands.forEach(function (command) {
                command.set("model", item);
            });
        }
    });
});
