﻿define("workspace/components/WorkspaceTree", [
    "dojo/_base/declare",
    "dojo/_base/lang",
    "epi",
    "epi/dependency",
    "epi-cms/component/PageTree",
    "epi-cms/dgrid/formatters",
    "workspace/commands/RemoveItem"
], function (declare, lang, epi, dependency, PageTree, formatters, RemoveItem) {

    return declare([PageTree], {
        sortKey: null,
        descending: false,
        store: null,

        postMixInProperties: function () {

            this.inherited(arguments);

            lang.mixin(this.defaultGridMixin, {
                columns: {
                    name: {
                        renderCell: function (object, value, node) {
                            node.innerHTML = formatters.contentItem(object.typeIdentifier, object.missingLanguageBranch, value);
                        }
                    },
                    status: {
                        className: "epi-width15",
                        renderCell: function (object, value, node) {
                            var cssClass = value == 3 ? "epi-iconCheckmark" : "epi-iconClose";
                            node.innerHTML = "<span class=\"dijitInline dijitIcon " + cssClass + "\"></span>";
                        }
                    },
                    menu: {
                        className: "epi-width10",
                        formatter: formatters.menu
                    }
                }
            });

            this.commands = [
                new RemoveItem({ store: this.store, contentQuery: this }),
                new RemoveItem({ store: this.store, contentQuery: this })
            ];
        },

        postCreate: function () {
            this.inherited(arguments);
            this.grid.contextMenu.addProvider(this);
        },

        fetchData: function () {
            var queryOptions = { ignore: ["query"] };
            if (this.sortKey) {
                queryOptions.sort = [{ attribute: this.sortKey, descending: this.descending }];
            }
            this.grid.set("queryOptions", queryOptions);

            var queryParameters = this.queryParameters || {};
            queryParameters.query = this.queryName;
            this.grid.set("query", queryParameters);
        },

        _onSelect: function (e) {
            var item = e.rows[0].data;

            this.commands.forEach(function (command) {
                command.set("model", item);
            });
        }
    });
});
