﻿function NodeViewModel(data) {
    var self = this;

    self.find = function (id) {
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }

        return null;
    };

    self.data = data;
    self.id = ecore.ko.observable(data.id);
    self.isExpanded = ecore.ko.observable(false);
    self.selected = ecore.ko.observable(data.selected || ecore.viewModel.findAsset(data.id));
    self.name = ecore.ko.observable(data.name);
    self.notTranslated = ecore.ko.observable(false);

    self.nodes = ecore.ko.observableArray([]);
    self.selectAllTree = function () {
        self.selected(true);
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].selectAllTree();
    }

    self.setTranslationStatus = function () {
        if (ecore.viewModel.languageFilter()) {
            self.notTranslated(self.data.missingLanguages.indexOf(ecore.viewModel.languageFilter()) >= 0);
            //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
            return;
        }

        self.notTranslated(false);
        //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
    }

    self.setTranslationStatus();

    self.setTranslationStatusTree = function () {
        var selfNodes = self.nodes();
        self.setTranslationStatus();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].setTranslationStatusTree();
    }

    self.deSelectAllTree = function () {
        self.selected(false);
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].deSelectAllTree();
    }
    self.addAllTree = function () {
        ecore.viewModel.isLoading(true);
        var assets = ecore.treeStore.put(self.id() + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);
        if (assets.then) {
            assets.then(function (item) {
                ecore.viewModel.addAssetsFromTree(item.result);
                //ecore.viewModel.isLoading(false);
            });
        }

        self.selectAllTree();
    };
    self.removeAllTree = function () {
        ecore.viewModel.isLoading(true);
        var assets = ecore.treeStore.put(self.id() + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);
        if (assets.then) {
            assets.then(function (item) {
                ecore.viewModel.removeAssetsFromTree(item.result);
                //ecore.viewModel.isLoading(false);
            });
        }
        self.deSelectAllTree();
    };
    self.hasNodes = ecore.ko.observable(data.hasChildren);
    self.childrenLoaded = ecore.ko.observable(false);
    self.selected.subscribe(function (value) {
        ecore.viewModel.isAddFromPagesTree(true);
        if (value) {
            ecore.viewModel.addAssetFromTree(self);
        }
        else
            ecore.viewModel.removeAssetFromTree(self);
    });

    self.appendChildren = function (nodes) {
        self.childrenLoaded(true);

        for (var i = 0; i < nodes.length; i++)
            self.nodes.push(new NodeViewModel(nodes[i]));

    };

    if (data.nodes)
        self.appendChildren(data.nodes);

    self.toggleVisibility = function () {
        self.isExpanded(!self.isExpanded());

        if (self.isExpanded() && !self.childrenLoaded()) { // load new children
            self.childrenLoaded(true);
            ecore.loadChildNodes(self.id(), self);
        }
    };
};

function BlockNodeViewModel(data) {
    var self = this;

    self.find = function (id) {
        var selfNodes = self.blockNodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }

        return null;
    };

    self.data = data;
    self.id = ecore.ko.observable(data.id);
    self.isExpanded = ecore.ko.observable(false);
    self.selected = ecore.ko.observable(data.selected || ecore.viewModel.findBlockAsset(data.id));
    self.selectable = ecore.ko.observable(data.selectable);
    self.name = ecore.ko.observable(data.name);
    self.notTranslated = ecore.ko.observable(false);

    self.blockNodes = ecore.ko.observableArray([]);

    self.selectAllTree = function () {
        self.selected(true);
        var selfNodes = self.blockNodes();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].selectAllTree();
    }

    self.setTranslationStatus = function () {
        if (ecore.viewModel.languageFilter()) {
            self.notTranslated(self.data.missingLanguages.indexOf(ecore.viewModel.languageFilter()) >= 0);
            //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
            return;
        }

        self.notTranslated(false);
        //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
    }

    self.setTranslationStatus();

    self.setTranslationStatusTree = function () {
        var selfNodes = self.blockNodes();
        self.setTranslationStatus();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].setTranslationStatusTree();
    }

    self.deSelectAllTree = function () {
        self.selected(false);
        var selfNodes = self.blockNodes();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].deSelectAllTree();
    }
    self.addAllTree = function () {
        ecore.viewModel.isLoading(true);
        var blockAssets = ecore.treeBlockStore.put(self.id() + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);
        if (blockAssets.then) {
            blockAssets.then(function (item) {
                ecore.viewModel.addBlockAssetsFromTree(item.result);
                //ecore.viewModel.isLoading(false);
            });
        }

        self.selectAllTree();
    };
    self.removeAllTree = function () {
        ecore.viewModel.isLoading(true);
        var assets = ecore.treeBlockStore.put(self.id() + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);
        if (assets.then) {
            assets.then(function (item) {
                ecore.viewModel.removeBlockAssetsFromTree(item.result);
                //ecore.viewModel.isLoading(false);
            });
        }
        self.deSelectAllTree();
    };
    self.hasNodes = ecore.ko.observable(data.hasChildren);
    self.childrenLoaded = ecore.ko.observable(false);
    self.selected.subscribe(function (value) {
        ecore.viewModel.isAddFromPagesTree(false);

        if (value)
            ecore.viewModel.addBlockAssetFromTree(self);
        else
            ecore.viewModel.removeBlockAssetFromTree(self);
    });

    self.appendBlockChildren = function (blockNodes) {
        self.childrenLoaded(true);

        for (var i = 0; i < blockNodes.length; i++)
            self.blockNodes.push(new BlockNodeViewModel(blockNodes[i]));

    };

    self.toggleVisibility = function () {
        self.isExpanded(!self.isExpanded());

        if (self.isExpanded() && !self.childrenLoaded()) { // load new children
            self.childrenLoaded(true);
            ecore.loadChildBlockNodes(self.id(), self);
        }
    };

    //initialization of data
    if (data.blockNodes)
        self.appendBlockChildren(data.blockNodes);

};

function AssetViewModel() {
    var self = this;
    self.isLoading = ecore.ko.observable(false);
    self.isSaving = ecore.ko.observable(false);
    self.progress = ecore.ko.observable(0);
    self.progressTotal = ecore.ko.observable(0);
    self.assets = ecore.ko.observableArray([]);
    self.nodes = ecore.ko.observableArray([]);
    self.blockAssets = ecore.ko.observableArray([]);
    self.blockNodes = ecore.ko.observableArray([]);
    self.editable = ecore.ko.observable(false);
    self.syncBlocksFromPages = ecore.ko.observable(true);
    self.syncNestedPageBlocksFromSelectedBlock = ecore.ko.observable(true);
    self.isAddFromPagesTree = ecore.ko.observable(false);


    self.languageFilter = ecore.ko.observable('');
    self.availableLanguages = ecore.ko.observableArray([]);
    self.search = ecore.ko.observable('');
    self.throttledSearch = ecore.ko.computed(self.search).extend({ throttle: 400 });
    self.searchResults = ecore.ko.observableArray([]);
    self.showSearchPane = ecore.ko.observable(false);

    self.languageFilter.subscribe(function (newValue) {
        var length = self.nodes().length;
        for (var i = 0; i < length; i++) {
            self.nodes()[i].setTranslationStatusTree();
        }
        length = self.blockNodes().length;
        for (var i = 0; i < length; i++) {
            self.blockNodes()[i].setTranslationStatusTree();
        }
    });

    //block assets
    self.addBlockAssetsFromTree = function (blockNodes) {
        var length = blockNodes.length;

        self.progressTotal(length);
        if (length == 0) {
            ecore.viewModel.isLoading(false);
            return;
        }
        var index = 0;
        var processBlocks = function () {
            for (; index < length; index++) {
                var toProcess = blockNodes[index];
                self.addBlockAssetFromTree(toProcess, true);
                if (index + 1 < length && index % 25 == 0) {
                    setTimeout(processBlocks, 100);
                    self.progress(index);
                    index++;
                    break;
                }
                if (index + 1 == length) {
                    ecore.viewModel.isLoading(false);
                }
            }
        }
        processBlocks();
    };

    self.addBlockAssetFromTree = function (blockNode, isNotObservable, refererId) {

        var blockAsset = {};
        if (isNotObservable) {
            if (!blockNode.selectable) return;
            blockAsset.id = blockNode.id;
            blockAsset.name = blockNode.name;
            blockAsset.isDeleted = false;

        } else {
            if (!blockNode.selectable()) return;
            blockAsset.id = blockNode.id();
            blockAsset.name = blockNode.name();
            blockAsset.isDeleted = false;
        }

        blockAsset.referers = blockAsset.referers || [];

        if (refererId && blockAsset.referers.indexOf(refererId) === -1) {
            blockAsset.referers.push(refererId);
        }

        blockAsset.status = '';

        var addedBlockAsset = ecore.viewModel.findBlockAsset(blockAsset.id);
        var isAddFromPagesTree = self.isAddFromPagesTree();
        if (!addedBlockAsset) {
            self.blockAssets.push(blockAsset);
            addedBlockAsset = blockAsset;
            if (!isAddFromPagesTree && self.syncNestedPageBlocksFromSelectedBlock()) {
                self.addNestedPages(blockAsset.id, true);
                if (blockNode.referencedBlocks) {
                    var id = blockNode.id ? blockNode.id : blockNode.id();
                    self.addReferencedBlocks(blockNode.referencedBlocks, id);
                }
            }
        } else {
            if (refererId && addedBlockAsset.referers.indexOf(refererId) === -1) {
                addedBlockAsset.referers.push(refererId);
            }
        }

        if (!refererId) return;

        //add the referer id if needed
        addedBlockAsset.referers = addedBlockAsset.referers || [];

        if (addedBlockAsset.referers.indexOf(refererId) === -1) {
            addedBlockAsset.referers.push(refererId);
        }
    };

    self.findBlockAsset = function (id) {
        var selfBlockAssets = self.blockAssets();
        for (var i = 0; i < selfBlockAssets.length; i++) {
            if (selfBlockAssets[i].id == id)
                return selfBlockAssets[i];
        }
        return null;
    };

    self.removeBlockAssetsFromTree = function (blockNodes) {
        var length = blockNodes.length;
        self.progressTotal(length);
        if (length == 0) {
            ecore.viewModel.isLoading(false);
            return;
        }
        var index = 0;
        var process = function () {
            for (; index < length; index++) {
                var toProcess = blockNodes[index];
                self.removeBlockAssetFromTree(toProcess, true);
                if (index + 1 < length && index % 25 == 0) {
                    setTimeout(process, 100);
                    self.progress(index);
                    index++;
                    break;
                }
                if (index + 1 == length) {
                    ecore.viewModel.isLoading(false);
                }
            }
        }
        process();
    };

    self.removeBlockAssetFromTree = function (blockNode, isNotObservable) {
        var assets = self.blockAssets();

        for (var i = 0; i < assets.length; i++) {
            if (assets[i].id.toString().replace('c', '') == (isNotObservable ? blockNode.id.toString().replace('c', '') : blockNode.id().toString().replace('c', ''))) {
                self.removeNestedPages(assets[i].id, true);
                self.removeBlockAsset(assets[i]);
            }
        }

    };

    self.removeBlockAsset = function (blockAsset) {
        var id = blockAsset.id ? blockAsset.id : blockAsset.id();
        self.blockAssets.remove(blockAsset);


        var projectId = ecore.getProjectId();

        var query = ecore.treeBlockStore.executeMethod("GetBlockNodes", "", { ids: id, projectId: projectId });

        if (query.then) {
            query.then(function (item) {
                if (item.result) {
                    var assets = self.blockAssets();
                    for (var i = 0; i < item.result.length; i++) {
                        for (var j = 0; j < item.result[i].referencedBlocks.length; j++) {
                            for (var k = 0; k < assets.length; k++) {
                                if (assets[k].id.toString().replace('c', '') == item.result[i].referencedBlocks[j].toString().replace('c', '')) {

                                    if (assets[k].referers.length <= 1) {
                                        self.deselectBlockAssetInTree(assets[k]);
                                    } else {
                                        for (var n = 0; n < self.blockAssets().length; n++) {
                                            var blockAsset = self.findBlockAsset(assets[k].id.toString().replace('c', ''));
                                            if (!blockAsset) {
                                                continue;
                                            }
                                            var index = blockAsset.referers.indexOf(item.result[i].id);
                                            if (index > -1) {
                                                blockAsset.referers.splice(index, 1);
                                            }
                                        }
                                    }

                                }
                            }
                        }
                    }
                }
            });
        }
    };

    self.deselectBlockAssetInTree = function (blockAsset) {
        var blockNode = self.findBlockNode(blockAsset.id);

        if (blockNode)
            blockNode.selected(false);
        //else
        self.removeBlockAsset(blockAsset);
    };

    self.findBlockNode = function (id) {
        var selfBlockNodes = self.blockNodes();
        for (var i = 0; i < selfBlockNodes.length; i++) {
            if (selfBlockNodes[i].id() == id)
                return selfBlockNodes[i];
            else {
                var result = selfBlockNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }

        return null;
    };

    self.addReferencedBlocks = function (referencedBlocks, refererId) {
        if (!referencedBlocks) return;

        var notFoundBlocks = [];
        for (var i = 0; i < referencedBlocks.length; i++) {
            var blockNode = self.findBlockNode(referencedBlocks[i]);
            if (!blockNode) {
                notFoundBlocks.push(referencedBlocks[i]);
                continue;
            }

            self.addBlockAssetFromTree(blockNode, false, refererId);

            if (blockNode != undefined && blockNode != null) {
                blockNode.selected(true);
            }
        }

        if (notFoundBlocks.length > 0) {
            var ids = notFoundBlocks.join(';');
            var projectId = ecore.getProjectId();

            var query = ecore.treeBlockStore.executeMethod("GetBlockNodes", "", { ids: ids, projectId: projectId });

            if (query.then) {
                query.then(function (item) {
                    if (item.result) {
                        for (var i = 0; i < item.result.length; i++) {
                            item.result[i].selected = true;
                            self.addBlockAssetFromTree(item.result[i], true, refererId);

                            var relatedNode = self.find(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                });
            }
        }
    };

    self.removeReferencedBlocks = function (referencedBlocks, refererId) {
        if (!referencedBlocks) return;

        for (var i = 0; i < referencedBlocks.length; i++) {
            var blockAsset = self.findBlockAsset(referencedBlocks[i]);
            if (!blockAsset) {
                continue;
            }

            //check the referers
            blockAsset.referers = blockAsset.referers || [];
            var refererIndex = blockAsset.referers.indexOf(refererId);

            //remove our referer
            if (refererIndex !== -1) {
                blockAsset.referers.splice(refererIndex, 1);
            }

            if (!blockAsset.referers.length) {
                //no more referers - remove it
                self.removeBlockAsset(blockAsset);
            }
        }
    }

    //do the search
    self.throttledSearch.subscribe(function (newValue) {
        if (newValue.length <= 1) return;

        //console.log("Do Search for: " + newValue);

        ecore.search(newValue);
    });


    //page/commerce assets
    self.addAssetFromTree = function (node, isNotObservable) {
        if (!isNotObservable) {
            var asset = self.findAsset(node.id());
            if (!asset) {
                self.assets.push({ id: node.id(), name: node.name(), status: '', isDeleted: false, referers: [], isNotObservable: false });

            }

            if (self.syncBlocksFromPages()) {
                self.addReferencedBlocks(node.data.referencedBlocks, node.id());
                self.addNestedPages(node.id(), false);
            }

        } else {
            if (!ecore.viewModel.findAsset(node.id)) {
                self.assets.push({ id: node.id, name: node.name, status: '', isDeleted: false, referers: [], isNotObservable: isNotObservable });

                if (self.syncBlocksFromPages()) {
                    self.addReferencedBlocks(node.referencedBlocks, node.id);
                    self.addNestedPages(node.id, false);
                }
            }
            else if (ecore.viewModel.findAsset(node.id) && self.syncBlocksFromPages()) {
                self.addReferencedBlocks(node.referencedBlocks, node.id);
                self.addNestedPages(node.id, false);
            }
        }
    };

    self.addNestedPages = function (id, isAddedFromBlock) {
        var projectId = ecore.getProjectId();
        ecore.viewModel.isLoading(true);
        var query = ecore.treeStore.executeMethod("GetPageNodes", "", { ids: id, projectId: projectId });

        if (query.then) {
            query.then(function (item) {
                if (item.result) {

                    for (var i = 0; i < item.result.length; i++) {
                        var asset = self.findAsset(item.result[i].id);
                        if (!asset) {
                            self.assets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = self.find(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }

                        var addedPageAsset = self.findAsset(item.result[i].id);
                        addedPageAsset.referers = addedPageAsset.referers || [];

                        if (item.result[i].id != id && addedPageAsset.referers.indexOf(id) === -1) {
                            addedPageAsset.referers.push(id);
                        }

                        if (isAddedFromBlock) {
                            self.addReferencedBlocks(item.result[i].referencedBlocks, item.result[i].id);
                        }
                    }
                    ecore.viewModel.isLoading(false);
                }
            });

            ecore.viewModel.isLoading(false);
        }
    };

    self.addAssetsFromTree = function (nodes) {
        var length = nodes.length;
        self.progressTotal(length);
        if (length == 0) {
            ecore.viewModel.isLoading(false);
            return;
        }
        var index = 0;
        var process = function () {
            ecore.viewModel.isLoading(true);
            for (; index < length; index++) {
                var toProcess = nodes[index];
                self.addAssetFromTree(toProcess, true);
                if (index + 1 < length && index % 25 == 0) {
                    setTimeout(process, 100);
                    self.progress(index);
                    index++;
                    break;
                }
                if (index + 1 == length) {
                    ecore.viewModel.isLoading(false);
                }
            }
        }
        process();
    };

    self.removeAssetsFromTree = function (nodes) {
        var length = nodes.length;
        self.progressTotal(length);
        if (length == 0) {
            ecore.viewModel.isLoading(false);
            return;
        }
        var index = 0;
        var process = function () {
            for (; index < length; index++) {
                var toProcess = nodes[index];
                self.removeAssetFromTree(toProcess, true);
                if (index + 1 < length && index % 25 == 0) {
                    setTimeout(process, 100);
                    self.progress(index);
                    index++;
                    break;
                }
                if (index + 1 == length) {
                    ecore.viewModel.isLoading(false);
                }
            }
        }
        process();
    };

    self.removeNestedPages = function (id, isRemoveFromBlock) {
        var projectId = ecore.getProjectId();
        ecore.viewModel.isLoading(true);
        var query = ecore.treeStore.executeMethod("GetPageNodes", "", { ids: id, projectId: projectId });

        if (query.then) {
            query.then(function (item) {
                if (item.result) {
                    // var assets = self.assets();

                    for (var i = 0; i < item.result.length; i++) {
                        var asset = self.findAsset(item.result[i].id);
                        if (asset) {
                            if (asset.referers) {
                                if (asset.referers.length <= 1) {
                                    self.deselectInTree(asset);
                                    self.removeAsset(asset);
                                } else {

                                    var index = asset.referers.indexOf(id);
                                    if (index > -1) {
                                        asset.referers.splice(index, 1);
                                    }
                                }
                            } else {
                                self.deselectInTree(asset);
                                self.removeAsset(asset);
                            }

                        }

                        if (isRemoveFromBlock) {
                            //self.addReferencedBlocks(item.result[i].referencedBlocks, item.result[i].id);
                            self.removeReferencedBlocks(item.result[i].referencedBlocks, item.result[i].id);
                        }


                    }
                    var blockAssets = self.blockAssets();

                    for (var i = 0; i < blockAssets.length; i++) {
                        var blockAsset = self.findBlockAsset(blockAssets[i].id);
                        if (!blockAsset) {
                            continue;
                        }

                        //check the referers
                        blockAsset.referers = blockAsset.referers || [];
                        var refererIndex = blockAsset.referers.indexOf(id);

                        //remove our referer
                        if (refererIndex !== -1) {
                            blockAsset.referers.splice(refererIndex, 1);
                            if (!blockAsset.referers.length) {
                                //no more referers - remove it
                                self.deselectBlockAssetInTree(blockAsset);
                                self.removeBlockAsset(blockAsset);
                            }
                        }


                    }
                    ecore.viewModel.isLoading(false);
                } else {
                    var blockAssets = self.blockAssets();

                    for (var j = 0; j < blockAssets.length; j++) {
                        //console.log(blockAssets[j]);

                        if (blockAssets[j].referers && blockAssets[j].referers.length > 0)
                            for (var k = 0; k < blockAssets[j].referers.length; k++) {
                                if (blockAssets[j].referers[k].toString().replace('c', '') == id) {
                                    //self.removeBlockAsset(blockAssets[k]);
                                    //self.deselectBlockAssetInTree(blockAssets[k]);
                                }
                            }
                    }

                    ecore.viewModel.isLoading(false);
                }

                ecore.viewModel.isLoading(false);
            });
        }

        var blockAssets = self.blockAssets();
        for (var i = 0; i < blockAssets.length; i++) {
            var blockAsset = self.findBlockAsset(blockAssets[i].id);
            if (!blockAsset) {
                continue;
            }

            //check the referers
            blockAsset.referers = blockAsset.referers || [];
            var refererIndex = blockAsset.referers.indexOf(id);

            //remove our referer
            if (refererIndex !== -1) {
                blockAsset.referers.splice(refererIndex, 1);

                if (!blockAsset.referers.length) {
                    //no more referers - remove it
                    self.deselectBlockAssetInTree(blockAsset);
                    self.removeBlockAsset(blockAsset);

                }
            }


        }

        for (var i = 0; i < blockAssets.length; i++) {
            var blockAsset = self.findBlockAsset(blockAssets[i].id);
            if (!blockAsset) {
                continue;
            }
            blockAsset.referers = blockAsset.referers || [];
            var isRemove = true;
            if (blockAsset.referers != null && blockAsset.referers.length >= 1) {
                for (var j = 0; j < blockAsset.referers.length; j++) {
                    var blockAssetRef = self.findBlockAsset(blockAsset.referers[j]);
                    var asset = self.findAsset(blockAsset.referers[j]);
                    if (blockAssetRef || asset) {
                        isRemove = false;
                        break;
                    }
                }
                if (isRemove) {
                    self.deselectBlockAssetInTree(blockAsset);
                    self.removeBlockAsset(blockAsset);
                }
            }
        }

        var selfAssets = self.assets();
        for (var i = 0; i < selfAssets.length; i++) {
            if (selfAssets[i].isNotObservable && (!selfAssets[i].referers || selfAssets[i].referers.length == 0))
                return selfAssets[i];
        }


    };


    self.find = function (id) {
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }

        return null;
    };
    self.findAsset = function (id) {
        var selfAssets = self.assets();
        for (var i = 0; i < selfAssets.length; i++) {
            if (selfAssets[i].id == id)
                return selfAssets[i];
        }
        return null;
    };

    self.removeAssetFromTree = function (node, isNotObservable) {
        var assets = self.assets();

        for (var i = 0; i < assets.length; i++) {

            if (assets[i].id.toString() == (isNotObservable ? node.id.toString() : node.id().toString())) {

                self.removeAsset(assets[i]);
                self.removeNestedPages(isNotObservable ? node.id.toString() : node.id().toString());
            }
        }
    };
    self.deselectInTree = function (asset) {
        var node = self.find(asset.id);

        if (node)
            node.selected(false);
        //else
        self.removeAsset(asset);
    };
    self.removeAsset = function (asset) {
        self.assets.remove(asset);

        if (self.syncBlocksFromPages()) {
            var node = self.find(asset.id);
            if (!node) return;
            self.removeReferencedBlocks(node.data.referencedBlocks, asset.id);
        }
    };
    self.save = function () {
        ecore.saveAssets();
    };

    self.close = function () {
        ecore.closeDialog();
    };

    self.appendChildren = function (nodes) {
        for (var i = 0; i < nodes.length; i++)
            self.nodes.push(new NodeViewModel(nodes[i]));
    };

    self.appendSearchResult = function (nodes) {
        self.searchResults([]);

        for (var i = 0; i < nodes.length; i++)
            self.searchResults.push(new NodeViewModel(nodes[i]));

        self.showSearchPane(true);
    };

    self.hideSearch = function (event) {
        var e = event || window.event;
        if (e.tag === "click_in_search_pane") return;

        self.showSearchPane(false);
    };

    self.preventDefault = function (event) {
        var e = event || window.event;
        e.tag = "click_in_search_pane";
    };

    self.appendBlockChildren = function (blockNodes) {
        for (var i = 0; i < blockNodes.length; i++)
            self.blockNodes.push(new BlockNodeViewModel(blockNodes[i]));
    };
}

window["ecore"] = {
    epieditor: null,
    dataStore: null,
    treeStore: null,
    treeBlockStore: null,
    searchStore: null,
    viewModel: null,
    ko: null,
    _projectId: -1,
    dialog: null,
    inited: false,

    //registry.get("workspace.translationprojecteditorstore"),
    //registry.get("workspace.translationprojecttreestore"),
    //registry.get("workspace.translationsearchitemsstore"),

    init: function (epieditor, dataStore, treeStore, treeBlockStore, searchStore, ko) {
        var self = this;
        self.dataStore = dataStore;
        self.epieditor = epieditor;
        self.treeStore = treeStore;
        self.treeBlockStore = treeBlockStore;
        self.searchStore = searchStore;
        self.ko = ko;

        if (!self.dialog)
            self.dialog = new dijit.Dialog({ content: dojo.cache("workspace.editors", "MultiSelectEditorLightbox.html?v=" + new Date().getTime()), style: 'width: 90%; height: 95%', title: 'Assets in Project' });

        if (!self.inited) {
            try {
                self.initTabs();
                self.createNewModel();
                self.bindModel();
                self.inited = true;
            } catch (err) {
                console.error(err.message);
            }


        }
    },

    getProjectId: function () {
        if (this._projectId == -1)
            this._projectId = $('#lb-project-id').val();

        return this._projectId;
    },

    createNewModel: function () {
        this.viewModel = new AssetViewModel();
    },

    showDialog: function () {
        var self = this;
        self.emptyModel();
        self.loadAssets();
        self.loadRootNodes();
        self.loadRootBlockNodes();
        self.dialog.show();
        self.loadDialog(this.dialog);

        //console.log('Loading project: ' + self.getProjectId());
    },

    loadRootNodes: function () {
        this.viewModel.nodes([]);
        this.loadChildNodes(-1, this.viewModel);
    },

    loadChildNodes: function (parentID, model) {
        var self = this;
        var parameters = parentID + ";" + ecore.getProjectId();
        if (self.viewModel.languageFilter()) {
            parameters += ";" + self.viewModel.languageFilter();
        }
        var query = self.treeStore.get(parameters + "?" + Math.random() * 9999999999);

        if (query.then) {
            query.then(function (item) {

                var assets = self.viewModel.assets();
                for (var i = 0; i < item.result.length; i++) {
                    var isSelectedAsset = false;
                    for (var j = 0; j < assets.length; j++) {

                        if (item.result[i].id == assets[j].id) {
                            item.result.selected = true;
                            isSelectedAsset = true;
                            break;
                        }
                    }

                    if (!isSelectedAsset) {
                        item.result.selected = false;
                    }
                }
                model.appendChildren(item.result);

                //var test = self.viewModel.assets();
            });
        } else {
            model.appendChildren(query.result);
        }
    },

    loadRootBlockNodes: function () {
        this.viewModel.blockNodes([]);
        this.loadChildBlockNodes(-1, this.viewModel);
    },

    loadChildBlockNodes: function (parentID, model) {
        var self = this;
        var parameters = parentID + ";" + ecore.getProjectId();
        if (self.viewModel.languageFilter()) {
            parameters += ";" + self.viewModel.languageFilter();
        }

        var query = self.treeBlockStore.get(parameters + "?" + Math.random() * 9999999999);

        if (query.then) {
            query.then(function (item) {

                var blockAssets = self.viewModel.blockAssets();
                for (var i = 0; i < item.result.length; i++) {
                    var isSelectedAsset = false;
                    for (var j = 0; j < blockAssets.length; j++) {

                        if (item.result[i].id == blockAssets[j].id) {
                            item.result.selected = true;
                            isSelectedAsset = true;
                            break;
                        }
                    }

                    if (!isSelectedAsset) {
                        item.result.selected = false;
                    }
                }

                model.appendBlockChildren(item.result);
            });
        } else {
            model.appendBlockChildren(query.result);
        }

    },

    saveAssets: function () {

        var self = this;
        var data = self.getProjectId();
        var assets = self.viewModel.assets();

        for (var i = 0; i < assets.length; i++)
            data += '_' + assets[i].id;

        var blockAssets = self.viewModel.blockAssets();
        for (var i = 0; i < blockAssets.length; i++)
            data += '_' + blockAssets[i].id;

        ecore.viewModel.isSaving(true);
        self.dataStore.put(data).then(function (response) {
            ecore.closeDialog();
            ecore.viewModel.isSaving(false);
        });
    },

    closeDialog: function () {
        this.dialog.hide();
    },

    emptyModel: function () {
        this._projectId = -1;
        this.viewModel.assets([]);
        this.viewModel.nodes([]);
        this.viewModel.search('');
        this.viewModel.languageFilter('');
    },

    bindModel: function () {
        this.ko.applyBindings(this.viewModel, document.getElementById("lb-editor"));
    },

    unbindModel: function () {
        var self = this;
        var node = jQuery("#lb-editor");
        node.find("*").each(function () {
            jQuery(this).unbind();
        });

        ko.cleanNode(document.getElementById("lb-editor"));
    },

    getHeight: function () {
        var total = jQuery('.dijitDialog').outerHeight();
        var title = jQuery('.dijitDialogTitleBar').outerHeight();
        return total - title;
    },

    loadDialog: function (dialog) {
        var self = this;
        var size = self.getHeight();
        jQuery('.dijitDialogPaneContent').css('height', size + 'px');
        jQuery('.dijitDialogPaneContent').css('padding', '0px');
        jQuery('.lb-pane').css('max-height', (size - 145) + 'px');
        jQuery('.lb-pane').css('height', (size - 145) + 'px');
        self.dialog = dialog;
    },

    initTabs: function () {
        var self = this;
        var size = self.getHeight() - 145;

        var tc = new dijit.layout.TabContainer({
            style: "width: 100%; height: " + size + "px;",
            useMenu: false,
            useSlider: false,
            doLayout: false
        }, "lb-editor-tabContainer");


        var cp1 = new dijit.layout.ContentPane({
            title: "Pages",
            content: dojo.cache("workspace.editors", "MultiSelectEditorLightboxPages.html?v=" + new Date().getTime()),
            style: "width:100%; height: 100%;",
            selected: true,
            doLayout: false
        });
        tc.addChild(cp1);

        var cp2 = new dijit.layout.ContentPane({
            title: "Blocks",
            content: dojo.cache("workspace.editors", "MultiSelectEditorLightboxBlocks.html?v=" + new Date().getTime()),
            style: "width:100%; height: 100%;",
            doLayout: false
        });
        tc.addChild(cp2);

        tc.startup();
    },

    loadAssets: function () {
        var self = this;
        var query = self.dataStore.get(self.getProjectId() + "?" + Math.random() * 9999999999);


        if (query.then) {
            query.then(function (item) {
                self.viewModel.editable(item.result.editable);
                self.viewModel.assets(item.result.assets);
                self.viewModel.blockAssets(item.result.blockAssets);
                self.viewModel.availableLanguages(item.result.availableLanguages);
            });
        }
        else {
            self.viewModel.editable(query.result.editable);
            self.viewModel.assets(query.result.assets);
            self.viewModel.blockAssets(query.result.blockAssets);
            self.viewModel.availableLanguages(query.result.availableLanguages);
        }
    },

    search: function (searchString) {
        var self = this;
        var query = self.searchStore.get(self.getProjectId() + ";" + searchString + "?" + Math.random() * 9999999999);

        if (query.then) {
            query.then(function (item) {
                self.viewModel.appendSearchResult(item.result);
            });
        }
        else {
            self.viewModel.appendSearchResult(query.result);
        }
    }
};

