﻿define([

    // Dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/on",
    "dojox/html/entities",

    // Dijit
    "dijit/focus",
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/form/RadioButton",
    // EPi
    "epi/shell/widget/_ValueRequiredMixin"
], function (

    // Dojo
    array,
    declare,
    lang,
    domConstruct,
    on,
    entities,

    // Dijit
    focusUtil,
    _TemplatedMixin,
    _Widget,
    RadioButton,
    // EPi
    _ValueRequiredMixin

) {

        return declare([_Widget, _TemplatedMixin, _ValueRequiredMixin], {
            // tags:
            //      internal

            templateString: "<div class=\"dijit dijitReset dijitInline\"></div>",

            baseClass: "epi-checkBoxList",

            valueIsCsv: false,

            valueIsInclusive: true,

            value: null,

            constructor: function () {
                this._radios = [];
                this._labelClickHandles = [];
            },

            buildRendering: function () {
                this.inherited(arguments);


                var groups = {};
                for (var i = 0; i < this.selections.length; i++) {
                    var key = this.selections[i].text;
                    if (!groups[key]) {
                        groups[key] = [];
                    }
                    groups[key].push({ text: this.selections[i].text, value: this.selections[i].value });
                }
                var selectionGroup = [];

                for (key in groups) {
                    selectionGroup.push({ key: key, value: groups[key] });
                }

                for (var item in selectionGroup) {
                    var label = domConstruct.create("label", { "class": "epi-labelContainer" }, this.domNode);
                    label.textContent = selectionGroup[item].key;

                    array.forEach(selectionGroup[item].value, this._addRadioForItem, this);
                }

                //array.forEach(this.selections, this._addRadioForItem, this);
            },

            destroy: function () {
                var radio;
                while ((radio = this._radios.pop())) {
                    radio.destroyRecursive();
                }

                var labelClickHandle;
                while ((labelClickHandle = this._labelClickHandles.pop())) {
                    labelClickHandle.remove();
                }

                this.inherited(arguments);
            },

            focus: function () {
                // summary:
                //		Focus the widget.

                try {
                    if (this._radios.length > 0) {
                        focusUtil.focus(this._radios[0].domNode);
                    }
                } catch (e) {
                    /*quiet*/
                }
            },

            onChange: function (/*===== newValue =====*/) {
                // summary:
                //		Callback when this widget's value is changed.
                // tags:
                //		callback
            },

            onBlur: function () {
                // summary:
                //		Callback when this widget loses focus.
                // tags:
                //		callback
            },

            onFocus: function () {
                // summary:
                //		Callback when this widget gains focus.
                // tags:
                //		callback
            },

            _calculateValue: function () {
                var values = [];
                var storeChecked = this.valueIsInclusive;
                array.forEach(this._radios, function (radio) {
                    if (storeChecked === radio.checked) {
                        values.push(radio.params);
                    }
                });

                this._set("value", this.valueIsCsv ? values.join(",") : values);
            },

            _setValueAttr: function (value) {
                this._set("value", value);

                var values = [];
                if (value) {
                    values = this.valueIsCsv ? value.split(",") : value;
                }

                var checkValues = this.valueIsInclusive;
                var compareCheckboxValue = this._compareCheckboxValue;

                array.forEach(this._radios, function (radio) {
                    radio.set("checked", checkValues === array.some(values, function (v) {
                        return compareCheckboxValue(radio, v);
                    }));
                });
            },

            _compareCheckboxValue: function (radio, value) {
                // 'value' can be a different type from 'radio.value' so we stringify them before comparing.

                var checkboxValue = radio.params;

                if (checkboxValue === undefined || checkboxValue === null) {
                    return false;
                }

                return checkboxValue.name === value.name && checkboxValue.value === value.value;
            },

            _onBlur: function () {
                this.inherited(arguments);
                this.onBlur();
            },

            _addRadioForItem: function (item) {
                // Create a container for the label and radio.
                var container = domConstruct.create("div", { "class": "epi-checkboxContainer" }, this.domNode);

                // Add the radio to the container.
                var radio = new RadioButton({ value: item.value, name: item.text });
                radio.placeAt(container);

                this.own(
                    on(radio, "change", lang.hitch(this, function () {
                        this._calculateValue();
                        this.onChange(this.value);
                    }))
                );

                radio.set("readOnly", !!this.readOnly);

                this._radios.push(radio);

                // Add a label for the radio
                var label = domConstruct.create("label", { "for": radio.id }, container);
                label.textContent = item.value;
                this._labelClickHandles.push(on(label, "click", lang.hitch(this, function () {
                    focusUtil.focus(radio.domNode);
                })));
            }
        });
    });
