# Load the LionBridge assembly

# get the directory of this script file
$currentDirectory = Split-Path (Get-Project).FileName

# get the full path and file name of the App.config file in the same directory as this script
$webConfigFile = [IO.Path]::Combine($currentDirectory, 'web.config')

# initialize the xml object
$webConfig = New-Object XML
# load the config file as an xml object
$webConfig.Load($webConfigFile)

# Read connection string
$connectionString= $webConfig.configuration.connectionStrings.add | ? {$_.name -eq 'EPiServerDB'}

# Get connectionString
$connStrFinal = ""
foreach($s in  $connectionString.ConnectionString.Split(';'))
{
	# PS script doesn't work with connectionstring containing AttachDbFilename
    if($s.indexOf("AttachDbFilename")=-1)  
	{
        $connStrFinal = $connStrFinal + $s + ";"
    }
}


function Remove-LionbridgeConnector() {
	# DELETE LionbridgeConnector dlls EPiServer framework can not sync content types back to database
	if([System.IO.File]::Exists((Split-Path (Get-Project).FileName) + "\bin\LionBridgeConnector.Common.dll"))
	{
		remove-item ((Split-Path (Get-Project).FileName) + "\bin\LionBridgeConnector.Common.dll")
	}

	if([System.IO.File]::Exists((Split-Path (Get-Project).FileName) + "\bin\LionBridgeConnector.Episerver.dll"))
	{
		remove-item ((Split-Path (Get-Project).FileName) + "\bin\LionBridgeConnector.Episerver.dll")
	}

    # Call LionBridge code to clean workspace
    "Attempting to clean up LionbridgeConnector workspace"
	"ConnectionString: " + $connStrFinal
	$sqlRemoval = "
	Declare @page_id int
	SELECT @page_id = pkID  FROM tblContent WHERE fkContentTypeID = (SELECT pkID  FROM tblContentType WHERE ModelType LIKE '%LionbridgeConnector.EPiServer%' and Name = 'Workspace')
	IF @page_id IS NOT NULL 
		EXEC dbo.[editDeletePage] @page_id, 1
	END
	"

    $scon = New-Object System.Data.SqlClient.SqlConnection
    $scon.ConnectionString = $connStrFinal
    
    $cmd = New-Object System.Data.SqlClient.SqlCommand
    $cmd.Connection = $scon
    $cmd.CommandText = $sqlRemoval
    $cmd.CommandTimeout = 0
    
    $scon.Open()
    $cmd.ExecuteNonQuery()
    $scon.Dispose()
    $cmd.Dispose()

    # Uninstall the package
	"Attempting to remove LionbridgeConnector Package & its dependencies from the project"
    Uninstall-Package LionbridgeConnector
}

Export-ModuleMember Remove-LionbridgeConnector